// Imperative and Object-Oriented Programming Methodology
// Uppsala University - Copyright © 2020 Henrik Alfken(Schulze), Lars-Henrik Eriksson
#define _GNU_SOURCE
#include "yourcode.h"

static link_t *link_create(link_t *next, void *element)
{
  link_t *link = calloc(1,sizeof(link_t));
  link->next = next;
  link->element = element;
  return link;
}

/// Create a new sorted (empty) list
/// -- compare is a function that defines the sort order for the elements.
list_t *list_create(void_eq compare)
{
  list_t *list = calloc(1,sizeof(list_t));
  list->first = list->last = NULL; 
  list->eq = compare;
  return list;
}

/// Insert element into list according to the sort order.
void list_insert(list_t *list, void *element)
{
  link_t *link = link_create(NULL, element);
  if (list->first == NULL)
    {
      list->first = link;
    }
  else
    {
      list->last->next = link;
    }
  list->last = link;
}

/// Return the number of elements in the list.
int list_size(list_t *list)
{
  /// TODO
  /// Returnera antalet element i listan. Detta skall räknas ut
  /// genom iteration över samtliga länkar.
  int result = 0;
  for (link_t *cursor = list->first; cursor != NULL; cursor = cursor->next)
    { ++result; }
  return result;
}

/// Remove the list and its elements from memory.
void list_destroy(list_t *list)
{
  /// TODO
  /// Ta bort listan och alla länkar
  if (list != NULL)
    {
      link_t *cursor = list->first;
      link_t *deallocate_me;
      while (cursor != NULL)
        {
          deallocate_me = cursor;
          cursor = cursor->next;
          free(deallocate_me);
        }
      free(list);
    }
}

/// Merge the two lists into dest, leaving source empty
/// (note: implemented so that links move, not just elements!).
void list_remove_dups(list_t *list)
{
  /// TODO
  /// Alla länkar med dublettelement (utom det första) skall tas bort.
  /// Observera att när listan skapas anges en funktion för att
  /// jämföra element. Se driver.c för exempel.
  link_t *deallocate_me;
  for (link_t*outr_crsr=list->first; outr_crsr!=NULL; outr_crsr=outr_crsr->next)
    {
      // For the current outr_crsr, remove all copies coming later in the list:
      for (link_t* inr_crsr=outr_crsr; inr_crsr!=NULL; inr_crsr=inr_crsr->next)
        {
          if (inr_crsr->next != NULL \
              && strcmp(outr_crsr->element, inr_crsr->next->element) == 0)
            {   // In this case, inr_crsr->next holds a copy, so remove it:
              deallocate_me = inr_crsr->next;
              inr_crsr->next = inr_crsr->next->next;
              free(deallocate_me);
            }
        }
      // Don't forget to update the last pointer:
      if (outr_crsr->next == NULL) { list->last = outr_crsr; }
    }
}  

/// Returns a string representation of the list to be printed
/// No need to read or change this function!!
char *list_print(list_t *list, elem_to_string to_string)
{
  char *result = calloc(2048, sizeof(char));
  strcat(result, "[");
  
  for (link_t *cursor = list->first; cursor; cursor = cursor->next)
    {
      char *tmp = NULL;
      asprintf(&tmp,"%s, ",to_string?to_string(cursor->element):(char*)cursor->element);
      strcat(result, tmp);
      free(tmp);
    }

  int size = strlen(result);
  strcpy(result + (size > 1 ? size - 2 : size), "]");
  
  return result;
}
